/* gltest - small OpenGL tearing test program
 * Copyright (C) 2012-2013 Ralf Jung <post@ralfj.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "glwindow.h"
#include "glutil.h"

#include <GL/glx.h>
#include <string>


/** Make an X window fit for GL. You have to manage the X window yourself! */
class GLXBackend : public GLBackend {
public:
	/** Create a GL window class and find an appropriate visual */
	GLXBackend() : display(NULL), context(None) {}
	/** Fre all resources */
	virtual ~GLXBackend();
	
	/** Initialize GL backend, choose visual configuration and return the ID */
	virtual VisualID initialize(Display *display);
	
	/** create a GL context for the given window */
	virtual void createContext(Window window);
	
	/** Swap back and front buffers */
	virtual void swapBuffers() const;
	
	/** Set the swap interval */
	virtual void setSwapInterval(int i) const;
private:
	Display *display;
	GLXFBConfig config;
	GLXContext context;
	Window window;
	
	PFNGLXSWAPINTERVALMESAPROC funSwapIntervalMesa;
	PFNGLXSWAPINTERVALEXTPROC funSwapIntervalExt;
	PFNGLXCREATECONTEXTATTRIBSARBPROC funCreateContextAttribsARB;
	
	bool haveGLXExtension(const std::string &name);
	T_proc resolveGLXFunction(const char *extension, const char *function);
};
